// ****************************************************************************************
//   Java application to build structure library for small molecule ligands and their BS
//   
//                  developed by Hui Sun Lee (huisun.cadd@gmail.com)
// **************************************************************************************** 

import java.io.*;
import java.util.Vector;



public class BuildLigandBSStructuresLibrary{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 3 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: BuildLigandBSStructuresLibrary" );
			System.out.println( "             -opts  [PDB files list]" );
			System.out.println( "                          example:  " );
			System.out.println( "                            101m.pdb" );
			System.out.println( "                            102l.pdb" );	
			System.out.println( "                            102m.pdb" );
			System.out.println( "                            ...     " );	
			System.out.println( "                    [directory where PDB files are saved]" );
			System.out.println( "                    [directory where structure library will be built]" );
			
			return;
		}
		
		
		
		// ***** User Changeable Parameters *****
		
		int mMinNumHvyAtoms = 5;  // the minimum number of heavy atoms in a ligand
		int mMinNumBSRes = 7;     // the minimum number of binding site residues
		
		// *****************************



		String mPDBList = args[0];
		String mDirPDB = args[1];
		String mDirTarget = args[2];
		
		StructureHandler mSH = new StructureHandler();

		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;	
		
		DataOutputStream struct_out;

		Vector< Molecule > vMolecules_Proteins;
		Vector< Molecule > vMolecules_Ligands;
		Vector<Atom> vWaters;
		Vector<Atom> vMetals;
		
		String [] array;
		
		
        
		try{
			if( mDirPDB.charAt( mDirPDB.length() - 1 ) != '/' )
				mDirPDB = mDirPDB + "/";
			
			if( mDirTarget.charAt( mDirTarget.length() - 1 ) != '/' )
				mDirTarget = mDirTarget + "/";
				
				
				
			
			list_stream = new FileInputStream( new File( mPDBList ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );

			while( ( list_line = list_reader.readLine()) != null ){
				String PDB_file = list_line;
				String PDB_id = PDB_file.substring( 0, 4 );
				
				System.out.println( PDB_file );
				
				
				// *** read molecules ***
				vMolecules_Proteins = mSH.readBiomacromoleculesFromPDB( mDirPDB + PDB_file );
				vMolecules_Ligands = mSH.readHeteromoleculesFromPDB( mDirPDB + PDB_file );
				vWaters = mSH.readWatersFromPDB( mDirPDB + PDB_file );	
				vMetals = mSH.readMetalsFromPDB( mDirPDB + PDB_file );
				

		
				// *** remove DNA and RNA ***
				mSH.removeDNAAndRND( vMolecules_Proteins );	
				
				
				
				// *** Merge a covalently linked ligand or unnatural amino acids with the protein ***
				for( int i = 0 ; i < vMolecules_Proteins.size() ; i++ ){
					for( int j = 0 ; j < vMolecules_Ligands.size() ; j++ ){
						if( ( vMolecules_Ligands.size() != 0 ) && mSH.isConvalentlyLinkedLigand( vMolecules_Proteins.elementAt(i), vMolecules_Ligands.elementAt(j) ) ){
							for( int a = 0 ; a < vMolecules_Ligands.elementAt(j).atoms.size() ; a++ ){
								vMolecules_Proteins.elementAt(i).atoms.add( vMolecules_Ligands.elementAt(j).atoms.elementAt(a) );
							}
							
							vMolecules_Ligands.removeElementAt(j);
							j--;						
						}						
					}					
				}



				// *** remove small proteins ***
				for( int i = 0 ; i < vMolecules_Proteins.size() ; i++ ){
					if( mSH.getNumResidues( vMolecules_Proteins.elementAt(i) ) < 50 ){
						vMolecules_Proteins.removeElementAt(i);
						i--;
					}
				}
				
				
				
				// *** remove ions and too small molecules ***
				for( int i = 0 ; i < vMolecules_Ligands.size() ; i++ ){
					if( vMolecules_Ligands.elementAt(i).atoms.size() < mMinNumHvyAtoms ){
						vMolecules_Ligands.removeElementAt(i);
						i--;
					}					
				}
				
				
				
				// *** remove proteins that do not interacting with ligands ***
				mSH.removeNonInteractingProteinChains( vMolecules_Ligands, vMolecules_Proteins, 6.4 );
				
				
				
				
				// *** generate protein residue Hashtable and array ***
				for( int i = 0 ; i < vMolecules_Proteins.size() ; i++ ){
					vMolecules_Proteins.elementAt(i).setResIndex();
				}

				
				
				// *** write PDB structure files for ligands and their binding sites ***
				int index_BS = 0;
				
				for( int i = 0 ; i < vMolecules_Ligands.size() ; i++ ){
					for( int j = 0 ; j < vMolecules_Proteins.size() ; j++ ){
						Vector<Atom> vTemp_Waters = mSH.duplicateAtomVector( vWaters );
						mSH.removeNonInteractingAtomsWithMolecule( vTemp_Waters, vMolecules_Ligands.elementAt(i).atoms, 3.2 );
						mSH.removeNonInteractingAtomsWithMolecule( vTemp_Waters, vMolecules_Proteins.elementAt(j).atoms, 3.2 );
						
						Vector<Atom> vTemp_Metals = mSH.duplicateAtomVector( vMetals );
						mSH.removeNonInteractingAtomsWithMolecule( vTemp_Metals, vMolecules_Ligands.elementAt(i).atoms, 3.0 );
						mSH.removeNonInteractingAtomsWithMolecule( vTemp_Metals, vMolecules_Proteins.elementAt(j).atoms, 3.0 );
						
						vMolecules_Proteins.elementAt(j).resetBooleanArray();	
								
						mSH.selectContactingReceptorResidues( vMolecules_Ligands.elementAt(i).atoms, vMolecules_Proteins.elementAt(j), 4.5 );
						mSH.selectContactingReceptorResidues( vTemp_Waters, vMolecules_Proteins.elementAt(j), 3.2 );
						mSH.selectContactingReceptorResidues( vTemp_Metals, vMolecules_Proteins.elementAt(j), 3.0 );
						
						
						
						int num_BSRes = vMolecules_Proteins.elementAt(j).getNumSelectedResidue();
						
						if( num_BSRes >= mMinNumBSRes ){
							index_BS++;
							
							String chain_id = vMolecules_Proteins.elementAt(j).atoms.elementAt(0).chain_id;
							
							// *** write ligand structure ***
							mSH.writeMoleculeInPDBFormat( vMolecules_Ligands.elementAt(i).atoms, mDirTarget + PDB_id + chain_id + "-lig-" + index_BS + ".pdb" );
							
							// *** write binding site structure ***
							mSH.writeSelectedResiduesInPDBFormat( vMolecules_Proteins.elementAt(j), mDirTarget + PDB_id + chain_id + "-BS-" + index_BS + ".pdb" );
							
							// *** write protein structure ***
							mSH.writeMoleculeInPDBFormat( vMolecules_Proteins.elementAt(j).atoms, mDirTarget + PDB_id + chain_id + ".pdb" );
						}	
						
											
						
						vTemp_Waters.removeAllElements();
						vTemp_Metals.removeAllElements();
					}
				}
								
				vMolecules_Proteins.removeAllElements();
				vMolecules_Ligands.removeAllElements();
				vWaters.removeAllElements();
				vMetals.removeAllElements();
			}	

			list_reader.close();
			list_stream.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}		
}
	